@@ -155,6 +155,7 @@ makeFileReader(std::shared_ptr<const FileCompound> zimFile, offset_t offset, zsi
       mp_titleDirentAccessor = getTitleAccessor(titleOffset, titleSize, "Title index table");
       const_cast<bool&>(m_hasFrontArticlesIndex) = false;
     }
+    m_byTitleDirentLookup.reset(new ByTitleDirentLookup(mp_titleDirentAccessor.get()));
 
     readMimeTypes();
   }
@@ -308,60 +309,7 @@ makeFileReader(std::shared_ptr<const FileCompound> zimFile, offset_t offset, zsi
 
   FileImpl::FindxTitleResult FileImpl::findxByTitle(char ns, const std::string& title)
   {
-    log_debug("find article by title " << ns << " \"" << title << "\", in file \"" << getFilename() << '"');
-
-    entry_index_type l = 0;
-    entry_index_type u = entry_index_type(mp_titleDirentAccessor->getDirentCount());
-
-    if (l == u)
-    {
-      log_debug("namespace " << ns << " not found");
-      return { false, title_index_t(0) };
-    }
-
-    unsigned itcount = 0;
-    bool u_is_exact_match = false;
-    while (u - l > 1)
-    {
-      ++itcount;
-      entry_index_type p = l + (u - l) / 2;
-
-      auto d = getDirentByTitle(title_index_t(p));
-      int c = direntCompareTitle(ns, title, *d);
-
-      if (c <= 0) {
-        u = p;
-        u_is_exact_match = (c == 0);
-      } else {
-        l = p;
-      }
-    }
-
-    // We now have a range of 1 where:
-    // - l lower than what we search for (may be upper or equal if it was since the beginning)
-    // - u is upper or equal to what we search for.
-    //
-    // Let's check for l
-    auto d = getDirentByTitle(title_index_t(l));
-    int c = direntCompareTitle(ns, title, *d);
-
-    bool found;
-    entry_index_type ret_index;
-
-    if (c <= 0)
-    {
-      // If l is upper or equal, we have found a match (l), exact or not (c==0 ?)
-      found = (c==0);
-      ret_index = l;
-    } else {
-      // If l is lower, we have either a exact match (u if u is a exact match)
-      // or the upper bound of (virtual) searched range.  found = u_is_exact_match;
-      found = u_is_exact_match;
-      ret_index = u;
-    }
-
-    log_debug("article (" << d.getTitle() << ") " << found ? "":"not " << "found after " << itcount << " iterations in file \"" << getFilename() << "\"");
-    return { found, title_index_t(ret_index) };
+    return m_byTitleDirentLookup->find(ns, title);
   }
 
   FileCompound::PartRange
