@@ -288,9 +288,11 @@ namespace zim
       TINFO("write zimfile :");
       writeLastParts();
       ::close(data->out_fd);
+      data->out_fd = -1;
 
       TINFO("rename tmpfile to final one.");
       DEFAULTFS::rename(data->tmpFileName, data->zimName);
+      data->tmpFileName.clear();
 
       TINFO("finish");
     }
@@ -452,6 +454,7 @@ namespace zim
 
     CreatorData::~CreatorData()
     {
+      quitAllThreads();
       if (compCluster)
         delete compCluster;
       if (uncompCluster)
@@ -459,7 +462,12 @@ namespace zim
       for(auto& cluster: clustersList) {
         delete cluster;
       }
-      quitAllThreads();
+      if ( out_fd != - 1 ) {
+        ::close(out_fd);
+      }
+      if ( ! tmpFileName.empty() ) {
+        DEFAULTFS::removeFile(tmpFileName);
+      }
     }
 
     void CreatorData::quitAllThreads() {
@@ -487,21 +495,20 @@ namespace zim
         if (existing->isRedirect() && !dirent->isRedirect()) {
           unresolvedRedirectDirents.erase(existing);
           dirents.erase(ret.first);
+          existing->markRemoved();
           dirents.insert(dirent);
         } else {
-          std::cerr << "Impossible to add " << NsAsChar(dirent->getNamespace()) << "/" << dirent->getPath() << std::endl;
-          std::cerr << "  dirent's title to add is : " << dirent->getTitle() << std::endl;
-          std::cerr << "  existing dirent's title is : " << existing->getTitle() << std::endl;
-          return;
+          std::ostringstream ss;
+          ss << "Impossible to add " << NsAsChar(dirent->getNamespace()) << "/" << dirent->getPath() << std::endl;
+          ss << "  dirent's title to add is : " << dirent->getTitle() << std::endl;
+          ss << "  existing dirent's title is : " << existing->getTitle() << std::endl;
+          throw std::runtime_error(ss.str());
         }
       };
 
-      // If this is a redirect, we're done: there's no blob to add.
-      if (dirent->isRedirect())
-      {
+      if (dirent->isRedirect()) {
         unresolvedRedirectDirents.insert(dirent);
         nbRedirectItems++;
-        return;
       }
     }
 
